@@ -1,7 +1,7 @@
 /* mpfr_sub -- subtract two floating-point numbers
 
-Copyright 2001, 2002, 2003, 2004 Free Software Foundation.
-Contributed by the Spaces project, INRIA Lorraine.
+Copyright 2001, 2002, 2003, 2004, 2006, 2007 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -17,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Place, Fifth Floor, Boston,
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
 #include "mpfr-impl.h"
@@ -25,6 +25,9 @@ MA 02110-1301, USA. */
 int
 mpfr_sub (mpfr_ptr a, mpfr_srcptr b, mpfr_srcptr c, mp_rnd_t rnd_mode)
 {
+  MPFR_LOG_FUNC (("b[%#R]=%R c[%#R]=%R rnd=%d", b, b, c, c, rnd_mode),
+                 ("a[%#R]=%R", a, a));
+
   if (MPFR_ARE_SINGULAR (b,c))
     {
       if (MPFR_IS_NAN (b) || MPFR_IS_NAN (c))
