@@ -1,6 +1,7 @@
 /* Test file for mpfr_get_str.
 
-Copyright 1999, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
 This file is part of the MPFR Library.
 
@@ -16,8 +17,8 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-MA 02111-1307, USA. */
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+MA 02110-1301, USA. */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -57,7 +58,7 @@ check_small (void)
 
   mpfr_set_prec (x, 20);
   mpfr_set_ui (x, 2, GMP_RNDN);
-  mpfr_sub_one_ulp (x, GMP_RNDD);
+  mpfr_nexttozero (x);
   s = mpfr_get_str (NULL, &e, 4, 2, x, GMP_RNDU);
   if (strcmp (s, "20") || (e != 1))
     {
@@ -66,7 +67,7 @@ check_small (void)
       exit (1);
     }
   mpfr_free_str (s);
-  
+
   /* check n_digits=0 */
   mpfr_set_prec (x, 5);
   mpfr_set_ui (x, 17, GMP_RNDN);
@@ -74,7 +75,7 @@ check_small (void)
   mpfr_free_str (s);
   s = mpfr_get_str (NULL, &e, 36, 0, x, GMP_RNDN);
   mpfr_free_str (s);
-  
+
   mpfr_set_prec (x, 64);
   mpfr_set_si (x, -1, GMP_RNDN);
   mpfr_div_2exp (x, x, 63, GMP_RNDN); /* x = -2^(-63) */
@@ -96,7 +97,7 @@ check_small (void)
       exit (1);
     }
   mpfr_free_str (s);
-  
+
   /* check corner case ret!=0, j0!=0 in mpfr_get_str_aux */
   mpfr_set_prec (x, 100);
   mpfr_set_str_binary (x, "0.1001011111010001101110010101010101111001010111111101101101100110100011110110000101110110001011110000E-9");
@@ -108,7 +109,7 @@ check_small (void)
       exit (1);
     }
   mpfr_free_str (s);
-  
+
   /* check corner case exact=0 in mpfr_get_str_aux */
   mpfr_set_prec (x, 100);
   mpfr_set_str_binary (x, "0.1001001111101101111000101000110111111010101100000110010001111111011001101011101100001100110000000000E8");
@@ -120,7 +121,7 @@ check_small (void)
       exit (1);
     }
   mpfr_free_str (s);
-  
+
   for (p=4; p<=200; p++)
     {
       mpfr_set_prec (x, p);
@@ -134,8 +135,8 @@ check_small (void)
           exit (1);
         }
       mpfr_free_str (s);
-  
-      mpfr_add_one_ulp (x, GMP_RNDU);
+
+      mpfr_nexttoinf (x);
       s = mpfr_get_str (NULL, &e, 6, 2, x, GMP_RNDN);
       if (strcmp (s, "11") || (e != 2))
         {
@@ -147,7 +148,7 @@ check_small (void)
       mpfr_free_str (s);
 
       mpfr_set_str (x, "6.5", 10, GMP_RNDN);
-      mpfr_sub_one_ulp (x, GMP_RNDU);
+      mpfr_nexttozero (x);
       s = mpfr_get_str (NULL, &e, 6, 2, x, GMP_RNDN);
       if (strcmp (s, "10") || (e != 2))
         {
@@ -407,7 +408,7 @@ check_small (void)
       exit (1);
     }
   mpfr_free_str (s);
-  
+
   mpfr_set_str_binary (x, "11111001010011100101000001111111110001001001110110001E-136");
   s = mpfr_get_str (NULL, &e, 10, 9, x, GMP_RNDN);
   if (strcmp (s, "100693858") || e != -24)
@@ -415,7 +416,7 @@ check_small (void)
       printf ("Error in mpfr_get_str (27): s=%s e=%d\n", s, (int) e);
       exit (1);
     }
-    mpfr_free_str (s); 
+    mpfr_free_str (s);
   mpfr_set_str_binary (x, "10001000001110010110001011111011111011011010000110001E-110");
   s = mpfr_get_str (NULL, &e, 10, 14, x, GMP_RNDN);
   if (strcmp (s, "36923634350619") || e != -17)
@@ -963,7 +964,7 @@ check_special (int b, mp_prec_t p)
   int i, j;
   char s[MAX_DIGITS + 2], s2[MAX_DIGITS + 2], c;
   mp_exp_t e;
-  mp_rnd_t r;
+  int r;
   size_t m;
 
   /* check for invalid base */
@@ -982,7 +983,7 @@ check_special (int b, mp_prec_t p)
       for (r = 0; r < GMP_RND_MAX; r++)
         for (m= (i<3)? 2 : i-1 ; (int) m <= i+1 ; m++)
           {
-            mpfr_get_str (s, &e, b, m, x, r);
+            mpfr_get_str (s, &e, b, m, x, (mp_rnd_t) r);
             /* s should be 1 followed by (m-1) zeros, and e should be i+1 */
             if ((e != i+1) || strncmp (s, s2, m) != 0)
               {
@@ -996,7 +997,7 @@ check_special (int b, mp_prec_t p)
       for (r = 0; r < GMP_RND_MAX; r++)
         if (i >= 2)
           {
-            mpfr_get_str (s, &e, b, i, x, r);
+            mpfr_get_str (s, &e, b, i, x, (mp_rnd_t) r);
             /* should be i times (b-1) */
             c = (b <= 10) ? '0' + b - 1 : 'a' + (b - 11);
             for (j=0; (j < i) && (s[j] == c); j++);
@@ -1025,32 +1026,55 @@ check_special (int b, mp_prec_t p)
 }
 
 static void
-check_bug_base2k(void)
+check_bug_base2k (void)
 {
   /*
    * -2.63b22b55697e800000000000@130
    * +-0.1001100011101100100010101101010101011010010111111010000000000000000000000000+00000000000000000000001E522
   */
-  mpfr_t xx,yy,zz;
+  mpfr_t xx, yy, zz;
   char *s;
   mp_exp_t e;
 
-  mpfr_init2(xx,107);
-  mpfr_init2(yy,79);
-  mpfr_init2(zz,99);
+  mpfr_init2 (xx, 107);
+  mpfr_init2 (yy, 79);
+  mpfr_init2 (zz, 99);
 
-  mpfr_set_str(xx, "-1.90e8c3e525d7c0000000000000@-18", 16, GMP_RNDN);
-  mpfr_set_str(yy, "-2.63b22b55697e8000000@130", 16, GMP_RNDN);
-  mpfr_add(zz, xx, yy, GMP_RNDD);
-  s = mpfr_get_str(NULL, &e, 16, 0, zz, GMP_RNDN);
-  if (strcmp(s, "-263b22b55697e8000000000008"))
+  mpfr_set_str (xx, "-1.90e8c3e525d7c0000000000000@-18", 16, GMP_RNDN);
+  mpfr_set_str (yy, "-2.63b22b55697e8000000@130", 16, GMP_RNDN);
+  mpfr_add (zz, xx, yy, GMP_RNDD);
+  s = mpfr_get_str (NULL, &e, 16, 0, zz, GMP_RNDN);
+  if (strcmp (s, "-263b22b55697e8000000000008"))
     {
-      printf(
-"Error for get_str base 16\nGot %s expected -263b22b55697e8000000000008\n", s);
-      exit(1);
+      printf ("Error for get_str base 16\n"
+              "Got %s expected -263b22b55697e8000000000008\n", s);
+      exit (1);
     }
   mpfr_free_str (s);
-  mpfr_clears(xx,yy,zz,NULL);
+  mpfr_clears (xx, yy, zz, (void *) 0);
+}
+
+static void
+check_reduced_exprange (void)
+{
+  mpfr_t x;
+  char *s;
+  mp_exp_t emax, e;
+
+  emax = mpfr_get_emax ();
+  mpfr_init2 (x, 8);
+  mpfr_set_str (x, "0.11111111E0", 2, GMP_RNDN);
+  set_emax (0);
+  s = mpfr_get_str (NULL, &e, 16, 0, x, GMP_RNDN);
+  set_emax (emax);
+  if (strcmp (s, "ff0"))
+    {
+      printf ("Error for mpfr_get_str on 0.11111111E0 in base 16:\n"
+              "Got \"%s\" instead of \"ff0\".\n", s);
+      exit (1);
+    }
+  mpfr_free_str (s);
+  mpfr_clear (x);
 }
 
 #define ITER 1000
@@ -1071,6 +1095,7 @@ main (int argc, char *argv[])
 
   check_small ();
 
+  check_special (2, 2);
   for (i = 0; i < ITER; i++)
     {
       p = 2 + (randlimb () % (MAX_DIGITS - 1));
@@ -1087,7 +1112,7 @@ main (int argc, char *argv[])
       mpfr_set_exp (x, (e == -10) ? mpfr_get_emin () :
                     ((e == 10) ? mpfr_get_emax () : e));
       b = 2 + (randlimb () % 35);
-      r = RND_RAND();
+      r = (mp_rnd_t) RND_RAND();
       mpfr_get_str (s, &f, b, m, x, r);
     }
   mpfr_clear (x);
@@ -1111,7 +1136,9 @@ main (int argc, char *argv[])
   check3 (6.7274500420134077e-87, GMP_RNDU, "67275");
   check3 (6.7274500420134077e-87, GMP_RNDD, "67274");
 
-  check_bug_base2k();
+  check_bug_base2k ();
+  check_reduced_exprange ();
+
   tests_end_mpfr ();
   return 0;
 }
